use json_to_table::{json_to_table, Orientation};
use serde_json::json;
use tabled::{Style, Table};

#[test]
fn orientation_test_1() {
    let value = json!(
        {"widget": {
            "debug": "on",
            "window": {
                "title": "Sample Konfabulator Widget",
                "name": "main_window",
                "width": 500,
                "height": 500
            },
            "image": {
                "src": "Images/Sun.png",
                "name": "sun1",
                "hOffset": 250,
                "vOffset": 250,
                "alignment": "center"
            },
            "text": {
                "data": "Click Here",
                "size": 36,
                "style": "bold",
                "name": "text1",
                "hOffset": 250,
                "vOffset": 100,
                "alignment": "center",
                "onMouseUp": "sun1.opacity = (sun1.opacity / 100) * 90;"
            }
        }}
    );

    let table = json_to_table(&value)
        .set_style(Style::extended())
        .set_object_mode(Orientation::Horizontal)
        .to_string();

    assert_eq!(
        table,
        concat!(
            "╔══════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════╗\n",
            "║ widget                                                                                                                                                                                                                                                                   ║\n",
            "╠══════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════╣\n",
            "║ ╔═══════╦═══════════════════════════════════════════════════════════════╦══════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════╦═══════════════════════════════════════════════════════════════════╗ ║\n",
            "║ ║ debug ║ image                                                         ║ text                                                                                                                     ║ window                                                            ║ ║\n",
            "║ ╠═══════╬═══════════════════════════════════════════════════════════════╬══════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════╬═══════════════════════════════════════════════════════════════════╣ ║\n",
            "║ ║  on   ║ ╔═══════════╦═════════╦════════╦══════════════════╦═════════╗ ║ ╔═══════════╦══════════════╦═════════╦═════════╦═════════════════════════════════════════════╦══════╦════════╦═════════╗ ║ ╔════════╦═══════════════╦══════════════════════════════╦═══════╗ ║ ║\n",
            "║ ║       ║ ║ alignment ║ hOffset ║ name   ║ src              ║ vOffset ║ ║ ║ alignment ║ data         ║ hOffset ║ name    ║ onMouseUp                                   ║ size ║ style  ║ vOffset ║ ║ ║ height ║ name          ║ title                        ║ width ║ ║ ║\n",
            "║ ║       ║ ╠═══════════╬═════════╬════════╬══════════════════╬═════════╣ ║ ╠═══════════╬══════════════╬═════════╬═════════╬═════════════════════════════════════════════╬══════╬════════╬═════════╣ ║ ╠════════╬═══════════════╬══════════════════════════════╬═══════╣ ║ ║\n",
            "║ ║       ║ ║  center   ║  250    ║  sun1  ║  Images/Sun.png  ║  250    ║ ║ ║  center   ║  Click Here  ║  250    ║  text1  ║  sun1.opacity = (sun1.opacity / 100) * 90;  ║  36  ║  bold  ║  100    ║ ║ ║  500   ║  main_window  ║  Sample Konfabulator Widget  ║  500  ║ ║ ║\n",
            "║ ║       ║ ╚═══════════╩═════════╩════════╩══════════════════╩═════════╝ ║ ╚═══════════╩══════════════╩═════════╩═════════╩═════════════════════════════════════════════╩══════╩════════╩═════════╝ ║ ╚════════╩═══════════════╩══════════════════════════════╩═══════╝ ║ ║\n",
            "║ ╚═══════╩═══════════════════════════════════════════════════════════════╩══════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════╩═══════════════════════════════════════════════════════════════════╝ ║\n",
            "╚══════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════════╝",
        )
    );
}

#[test]
fn orientation_test_2() {
    let value = json!(
        {
            "glossary": {
                "title": "example glossary",
                "GlossDiv": {
                    "title": "S",
                    "GlossList": {
                        "GlossEntry": {
                            "ID": "SGML",
                            "SortAs": "SGML",
                            "GlossTerm": "Standard Generalized Markup Language",
                            "Acronym": "SGML",
                            "Abbrev": "ISO 8879:1986",
                            "GlossDef": {
                                "para": "A meta-markup language, used to create markup languages such as DocBook.",
                                "GlossSeeAlso": ["GML", "XML"]
                            },
                            "GlossSee": "markup"
                        }
                    }
                }
            }
        }
    );

    let table = json_to_table(&value)
        .set_style(Style::modern())
        .set_object_mode(Orientation::Horizontal)
        .set_array_mode(Orientation::Horizontal)
        .to_string();

    println!("{}", table);

    assert_eq!(
        table,
        concat!(
            "┌───────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┐\n",
            "│ glossary                                                                                                                                                                                                                                          │\n",
            "├───────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤\n",
            "│ ┌──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┬────────────────────┐ │\n",
            "│ │ GlossDiv                                                                                                                                                                                                                 │ title              │ │\n",
            "│ ├──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┼────────────────────┤ │\n",
            "│ │ ┌──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┬───────┐ │  example glossary  │ │\n",
            "│ │ │ GlossList                                                                                                                                                                                                    │ title │ │                    │ │\n",
            "│ │ ├──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┼───────┤ │                    │ │\n",
            "│ │ │ ┌──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┐ │  S    │ │                    │ │\n",
            "│ │ │ │ GlossEntry                                                                                                                                                                                               │ │       │ │                    │ │\n",
            "│ │ │ ├──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤ │       │ │                    │ │\n",
            "│ │ │ │ ┌─────────────────┬─────────┬────────────────────────────────────────────────────────────────────────────────────────────────────┬──────────┬────────────────────────────────────────┬────────┬────────┐ │ │       │ │                    │ │\n",
            "│ │ │ │ │ Abbrev          │ Acronym │ GlossDef                                                                                           │ GlossSee │ GlossTerm                              │ ID     │ SortAs │ │ │       │ │                    │ │\n",
            "│ │ │ │ ├─────────────────┼─────────┼────────────────────────────────────────────────────────────────────────────────────────────────────┼──────────┼────────────────────────────────────────┼────────┼────────┤ │ │       │ │                    │ │\n",
            "│ │ │ │ │  ISO 8879:1986  │  SGML   │ ┌───────────────────┬────────────────────────────────────────────────────────────────────────────┐ │  markup  │  Standard Generalized Markup Language  │  SGML  │  SGML  │ │ │       │ │                    │ │\n",
            "│ │ │ │ │                 │         │ │ GlossSeeAlso      │ para                                                                       │ │          │                                        │        │        │ │ │       │ │                    │ │\n",
            "│ │ │ │ │                 │         │ ├───────────────────┼────────────────────────────────────────────────────────────────────────────┤ │          │                                        │        │        │ │ │       │ │                    │ │\n",
            "│ │ │ │ │                 │         │ │ ┌───────┬───────┐ │  A meta-markup language, used to create markup languages such as DocBook.  │ │          │                                        │        │        │ │ │       │ │                    │ │\n",
            "│ │ │ │ │                 │         │ │ │  GML  │  XML  │ │                                                                            │ │          │                                        │        │        │ │ │       │ │                    │ │\n",
            "│ │ │ │ │                 │         │ │ └───────┴───────┘ │                                                                            │ │          │                                        │        │        │ │ │       │ │                    │ │\n",
            "│ │ │ │ │                 │         │ └───────────────────┴────────────────────────────────────────────────────────────────────────────┘ │          │                                        │        │        │ │ │       │ │                    │ │\n",
            "│ │ │ │ └─────────────────┴─────────┴────────────────────────────────────────────────────────────────────────────────────────────────────┴──────────┴────────────────────────────────────────┴────────┴────────┘ │ │       │ │                    │ │\n",
            "│ │ │ └──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘ │       │ │                    │ │\n",
            "│ │ └──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┴───────┘ │                    │ │\n",
            "│ └──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┴────────────────────┘ │\n",
            "└───────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘",
        )
    );
}

#[test]
fn orientation_visitor_test() {
    let value = json!(
        {"menu": {
            "header": "SVG Viewer",
            "items": [
                {"id": "Open"},
                {"id": "OpenNew", "label": "Open New"},
                null,
                {"id": "ZoomIn", "label": "Zoom In"},
                {"id": "ZoomOut", "label": "Zoom Out"},
                {"id": "OriginalView", "label": "Original View"},
                null,
                {"id": "Quality"},
                {"id": "Pause"},
                {"id": "Mute"},
                null,
                {"id": "Find", "label": "Find..."},
                {"id": "FindAgain", "label": "Find Again"},
                {"id": "Copy"},
                {"id": "CopyAgain", "label": "Copy Again"},
                {"id": "CopySVG", "label": "Copy SVG"},
                {"id": "ViewSVG", "label": "View SVG"},
                {"id": "ViewSource", "label": "View Source"},
                {"id": "SaveAs", "label": "Save As"},
                null,
                {"id": "Help"},
                {"id": "About", "label": "About Adobe CVG Viewer..."}
            ]
        }}
    );

    let mut i = false;
    let table = Table::from({
        let mut table = json_to_table(&value).set_mode_visitor(|_| {
            if i {
                i = false;
                Orientation::Horizontal
            } else {
                i = true;
                Orientation::Vertical
            }
        });

        table.set_style(Style::modern());

        table
    })
    .to_string();

    assert_eq!(
        table,
        concat!(
            "┌──────┬────────────────────────────────────────────────────────────────┐\n",
            "│ menu │ ┌──────────────┬─────────────────────────────────────────────┐ │\n",
            "│      │ │ header       │ items                                       │ │\n",
            "│      │ ├──────────────┼─────────────────────────────────────────────┤ │\n",
            "│      │ │  SVG Viewer  │ ┌─────────────────────────────────────────┐ │ │\n",
            "│      │ │              │ │ ┌────────┐                              │ │ │\n",
            "│      │ │              │ │ │ id     │                              │ │ │\n",
            "│      │ │              │ │ ├────────┤                              │ │ │\n",
            "│      │ │              │ │ │  Open  │                              │ │ │\n",
            "│      │ │              │ │ └────────┘                              │ │ │\n",
            "│      │ │              │ ├─────────────────────────────────────────┤ │ │\n",
            "│      │ │              │ │ ┌───────┬────────────┐                  │ │ │\n",
            "│      │ │              │ │ │ id    │  OpenNew   │                  │ │ │\n",
            "│      │ │              │ │ ├───────┼────────────┤                  │ │ │\n",
            "│      │ │              │ │ │ label │  Open New  │                  │ │ │\n",
            "│      │ │              │ │ └───────┴────────────┘                  │ │ │\n",
            "│      │ │              │ ├─────────────────────────────────────────┤ │ │\n",
            "│      │ │              │ │                                         │ │ │\n",
            "│      │ │              │ ├─────────────────────────────────────────┤ │ │\n",
            "│      │ │              │ │ ┌──────────┬───────────┐                │ │ │\n",
            "│      │ │              │ │ │ id       │ label     │                │ │ │\n",
            "│      │ │              │ │ ├──────────┼───────────┤                │ │ │\n",
            "│      │ │              │ │ │  ZoomIn  │  Zoom In  │                │ │ │\n",
            "│      │ │              │ │ └──────────┴───────────┘                │ │ │\n",
            "│      │ │              │ ├─────────────────────────────────────────┤ │ │\n",
            "│      │ │              │ │ ┌───────┬────────────┐                  │ │ │\n",
            "│      │ │              │ │ │ id    │  ZoomOut   │                  │ │ │\n",
            "│      │ │              │ │ ├───────┼────────────┤                  │ │ │\n",
            "│      │ │              │ │ │ label │  Zoom Out  │                  │ │ │\n",
            "│      │ │              │ │ └───────┴────────────┘                  │ │ │\n",
            "│      │ │              │ ├─────────────────────────────────────────┤ │ │\n",
            "│      │ │              │ │ ┌────────────────┬─────────────────┐    │ │ │\n",
            "│      │ │              │ │ │ id             │ label           │    │ │ │\n",
            "│      │ │              │ │ ├────────────────┼─────────────────┤    │ │ │\n",
            "│      │ │              │ │ │  OriginalView  │  Original View  │    │ │ │\n",
            "│      │ │              │ │ └────────────────┴─────────────────┘    │ │ │\n",
            "│      │ │              │ ├─────────────────────────────────────────┤ │ │\n",
            "│      │ │              │ │                                         │ │ │\n",
            "│      │ │              │ ├─────────────────────────────────────────┤ │ │\n",
            "│      │ │              │ │ ┌────┬───────────┐                      │ │ │\n",
            "│      │ │              │ │ │ id │  Quality  │                      │ │ │\n",
            "│      │ │              │ │ └────┴───────────┘                      │ │ │\n",
            "│      │ │              │ ├─────────────────────────────────────────┤ │ │\n",
            "│      │ │              │ │ ┌─────────┐                             │ │ │\n",
            "│      │ │              │ │ │ id      │                             │ │ │\n",
            "│      │ │              │ │ ├─────────┤                             │ │ │\n",
            "│      │ │              │ │ │  Pause  │                             │ │ │\n",
            "│      │ │              │ │ └─────────┘                             │ │ │\n",
            "│      │ │              │ ├─────────────────────────────────────────┤ │ │\n",
            "│      │ │              │ │ ┌────┬────────┐                         │ │ │\n",
            "│      │ │              │ │ │ id │  Mute  │                         │ │ │\n",
            "│      │ │              │ │ └────┴────────┘                         │ │ │\n",
            "│      │ │              │ ├─────────────────────────────────────────┤ │ │\n",
            "│      │ │              │ │                                         │ │ │\n",
            "│      │ │              │ ├─────────────────────────────────────────┤ │ │\n",
            "│      │ │              │ │ ┌────────┬───────────┐                  │ │ │\n",
            "│      │ │              │ │ │ id     │ label     │                  │ │ │\n",
            "│      │ │              │ │ ├────────┼───────────┤                  │ │ │\n",
            "│      │ │              │ │ │  Find  │  Find...  │                  │ │ │\n",
            "│      │ │              │ │ └────────┴───────────┘                  │ │ │\n",
            "│      │ │              │ ├─────────────────────────────────────────┤ │ │\n",
            "│      │ │              │ │ ┌───────┬──────────────┐                │ │ │\n",
            "│      │ │              │ │ │ id    │  FindAgain   │                │ │ │\n",
            "│      │ │              │ │ ├───────┼──────────────┤                │ │ │\n",
            "│      │ │              │ │ │ label │  Find Again  │                │ │ │\n",
            "│      │ │              │ │ └───────┴──────────────┘                │ │ │\n",
            "│      │ │              │ ├─────────────────────────────────────────┤ │ │\n",
            "│      │ │              │ │ ┌────────┐                              │ │ │\n",
            "│      │ │              │ │ │ id     │                              │ │ │\n",
            "│      │ │              │ │ ├────────┤                              │ │ │\n",
            "│      │ │              │ │ │  Copy  │                              │ │ │\n",
            "│      │ │              │ │ └────────┘                              │ │ │\n",
            "│      │ │              │ ├─────────────────────────────────────────┤ │ │\n",
            "│      │ │              │ │ ┌───────┬──────────────┐                │ │ │\n",
            "│      │ │              │ │ │ id    │  CopyAgain   │                │ │ │\n",
            "│      │ │              │ │ ├───────┼──────────────┤                │ │ │\n",
            "│      │ │              │ │ │ label │  Copy Again  │                │ │ │\n",
            "│      │ │              │ │ └───────┴──────────────┘                │ │ │\n",
            "│      │ │              │ ├─────────────────────────────────────────┤ │ │\n",
            "│      │ │              │ │ ┌───────────┬────────────┐              │ │ │\n",
            "│      │ │              │ │ │ id        │ label      │              │ │ │\n",
            "│      │ │              │ │ ├───────────┼────────────┤              │ │ │\n",
            "│      │ │              │ │ │  CopySVG  │  Copy SVG  │              │ │ │\n",
            "│      │ │              │ │ └───────────┴────────────┘              │ │ │\n",
            "│      │ │              │ ├─────────────────────────────────────────┤ │ │\n",
            "│      │ │              │ │ ┌───────┬────────────┐                  │ │ │\n",
            "│      │ │              │ │ │ id    │  ViewSVG   │                  │ │ │\n",
            "│      │ │              │ │ ├───────┼────────────┤                  │ │ │\n",
            "│      │ │              │ │ │ label │  View SVG  │                  │ │ │\n",
            "│      │ │              │ │ └───────┴────────────┘                  │ │ │\n",
            "│      │ │              │ ├─────────────────────────────────────────┤ │ │\n",
            "│      │ │              │ │ ┌──────────────┬───────────────┐        │ │ │\n",
            "│      │ │              │ │ │ id           │ label         │        │ │ │\n",
            "│      │ │              │ │ ├──────────────┼───────────────┤        │ │ │\n",
            "│      │ │              │ │ │  ViewSource  │  View Source  │        │ │ │\n",
            "│      │ │              │ │ └──────────────┴───────────────┘        │ │ │\n",
            "│      │ │              │ ├─────────────────────────────────────────┤ │ │\n",
            "│      │ │              │ │ ┌───────┬───────────┐                   │ │ │\n",
            "│      │ │              │ │ │ id    │  SaveAs   │                   │ │ │\n",
            "│      │ │              │ │ ├───────┼───────────┤                   │ │ │\n",
            "│      │ │              │ │ │ label │  Save As  │                   │ │ │\n",
            "│      │ │              │ │ └───────┴───────────┘                   │ │ │\n",
            "│      │ │              │ ├─────────────────────────────────────────┤ │ │\n",
            "│      │ │              │ │                                         │ │ │\n",
            "│      │ │              │ ├─────────────────────────────────────────┤ │ │\n",
            "│      │ │              │ │ ┌────────┐                              │ │ │\n",
            "│      │ │              │ │ │ id     │                              │ │ │\n",
            "│      │ │              │ │ ├────────┤                              │ │ │\n",
            "│      │ │              │ │ │  Help  │                              │ │ │\n",
            "│      │ │              │ │ └────────┘                              │ │ │\n",
            "│      │ │              │ ├─────────────────────────────────────────┤ │ │\n",
            "│      │ │              │ │ ┌───────┬─────────────────────────────┐ │ │ │\n",
            "│      │ │              │ │ │ id    │  About                      │ │ │ │\n",
            "│      │ │              │ │ ├───────┼─────────────────────────────┤ │ │ │\n",
            "│      │ │              │ │ │ label │  About Adobe CVG Viewer...  │ │ │ │\n",
            "│      │ │              │ │ └───────┴─────────────────────────────┘ │ │ │\n",
            "│      │ │              │ └─────────────────────────────────────────┘ │ │\n",
            "│      │ └──────────────┴─────────────────────────────────────────────┘ │\n",
            "└──────┴────────────────────────────────────────────────────────────────┘",
        )
    );
}
